library RexxUni;

{$CDecl+,OrgName+,I-,S-,Delphi+,Use32+}

Uses OS2Base, OS2Def, Rexx;

{$LINKER
  DESCRIPTION	"REXXUNI 1.2 - UNICODE API for REXX (c) 2001 by Max Alekseyev"
  DATA MULTIPLE NONSHARED

  EXPORTS
    UNIQUERYCP	  = UniQueryCp
    UNICPNAME	  = UniCpName
    UNIXLAT	  = UniXlat
    UNILOADFUNCS  = UniLoadFuncs
    UNIDROPFUNCS  = UniDropFuncs
}

function UniMapCpToUcsCp(ulCodePage:ulong;var ucsCodePage;size:ulong):ulong; external 'UCONV' name 'UniMapCpToUcsCp';
function UniCreateUconvObject(var cpname, UconvObject):ulong; external 'UCONV' name 'UniCreateUconvObject';
function UniUconvFromUcs(UconvObject:pointer;var src, ssize, dst, dsize, n):ulong; external 'UCONV' name 'UniUconvFromUcs';
function UniUconvToUcs(UconvObject:pointer;var src, ssize, dst, dsize, n):ulong; external 'UCONV' name 'UniUconvToUcs';
function UniFreeUconvObject(UconvObject:pointer):ulong; external 'UCONV' name 'UniFreeUconvObject';

Const
  FunctionTable : Array[1..5] of pChar
		= ('UniQueryCp','UniCpName','UniXlat','UniLoadFuncs','UniDropFuncs');
  DefaultStrLen = 256;			 // default length of RxString

type UniChar=SmallWord;

var p,d,j,t,Code:word;
    s:string;

Function UniLoadFuncs ( FuncName  : PChar;
			ArgC	  : ULong;
			Args	  : pRxString;
			QueueName : pChar;
			Var Ret   : RxString ) : ULong; export;
begin
  Ret.strLength := 0; UniLoadFuncs := 40;
  If ArgC = 0 then		     { Do not allow parameters }
  begin
    For j := Low(FunctionTable) to High(FunctionTable) do
	RexxRegisterFunctionDLL(FunctionTable[j], 'REXXUNI', FunctionTable[j]);
    UniLoadFuncs := 0;
  end;
end;

Function UniDropFuncs ( FuncName  : PChar;
			ArgC	  : ULong;
			Args	  : pRxString;
			QueueName : pChar;
			Var Ret   : RxString ) : ULong; export;
begin
  Ret.strLength := 0; UniDropFuncs := 40;
  If ArgC = 0 then		    { Do not allow parameters }
  begin
    For j := Low(FunctionTable) to High(FunctionTable) do
	RexxDeregisterFunction(FunctionTable[j]);
    UniDropFuncs := 0;
  end;
end;

function SystemCp:ULong;
var ArCP,PcCP:longint;
begin
  DosQueryCp(4,ArCP,PcCP);
  SystemCp := ArCP;
end;

function UniQueryCp( FuncName  : PChar;
		  ArgC	    : ULong;
		  Args	    : pRxString;
		  QueueName : pChar;
		  Var Ret   : RxString ) : ULong; export;
var s:string;
begin
  Ret.strLength:=0; UniQueryCp := 40;
  if ArgC=0 Then
  begin
    str(SystemCp,s);
    Ret.strLength := length(s);
    Move(s[1],Ret.strptr^,length(s));
    UniQueryCp := 0;
  end;
end;

function UniCpName(  FuncName  : PChar;
		  ArgC	    : ULong;
		  Args	    : pRxString;
		  QueueName : pChar;
		  Var Ret   : RxString ) : ULong; export;
var unistr:array[byte] of UniChar;
    cp, Code, u, v : integer;
    s: string;
    convobj, p, q: pointer;
begin
  Ret.strLength:=0; UniCpName := 40;
  if ArgC>1 Then Exit;

  UniCpName := 0; Code := 0;
  if ArgC=1 Then Val(Args^.strptr, cp, Code) else cp := SystemCp;
  if (Code<>0) or (UniMapCpToUcsCp(cp,s[1],127)<>0) or (UniCreateUconvObject(s[1],convobj)<>0) then Exit;
  UniFreeUconvObject(convobj);
  if (UniMapCpToUcsCp(0,unistr,256)<>0) or (UniCreateUconvObject(unistr,convobj)<>0) then Exit;
  s[0]:=#255; u := Pos(#0#0,s) shr 1;
  p := @s[1]; q := Ret.strptr;
  v := DefaultStrLen;
  Code := UniUconvFromUcs(convobj,p,u,q,v,Code);
  UniFreeUconvObject(convobj);
  if Code=0 Then Ret.strLength := DefaultStrLen - v;
end;

function UniXlat(  FuncName  : PChar;
		  ArgC	    : ULong;
		  Args	    : pRxString;
		  QueueName : pChar;
		  Var Ret   : RxString ) : ULong; export;
var unistr:array[byte] of UniChar;
    cp1, cp2, Code, u, v : integer;
    convobj, p, q, r : pointer;
type pbyte = ^byte;
begin
  Ret.strLength:=0; UniXlat := 40;
  if (ArgC<2) or (ArgC>3) Then Exit;

    Inc(Args);
    with Args^ do if strLength=0 Then cp1:=SystemCp else
    begin
      Val(strptr,cp1,Code);
      If Code<>0 Then Exit;
    end;

    Inc(Args);
    with Args^ do if (ArgC<3) or (strLength=0) Then cp2:=SystemCp else
    begin
      Val(strptr,cp2,Code);
      If Code<>0 Then Exit;
    end;
    Dec(Args);
    Dec(Args);

    if cp2<>0 Then
    begin
      if (UniMapCpToUcsCp(cp2, unistr, 256) <> 0) or (UniCreateUconvObject(unistr,convobj) <> 0) then Exit;
      u := Args^.strLength shl 1; v := u;
      GetMem(r,u); p:=r;
      Code := UniUconvToUcs(convobj,Args^.strptr,Args^.strLength,p,v,Code);
      dec(u,v);
      UniFreeUconvObject(convobj);
    end else
    begin
      r := Args^.strptr;
      u := Args^.strLength shr 1;
      Code := 0;
    end;

    if (Code=0) and (cp1<>0) Then
       Code := UniMapCpToUcsCp(cp1, unistr, 256) or UniCreateUconvObject(unistr,convobj);

    if Code=0 Then with Ret do
    begin
      v := u shl 1; strLength := v;
      if v>DefaultStrLen Then GetMem(strptr,v);
      if(cp1<>0) Then
      begin
	p:=r; q := strptr;
	Code := UniUconvFromUcs(convobj,p,u,q,v,Code);
	dec(strLength,v);
	UniFreeUconvObject(convobj);
      end else move(r^,strptr^,strLength);
      UniXlat := 0;
    end;

    if cp2<>0 Then FreeMem(r);
end;

initialization
end.

